"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkCreateNightwatchVersion = exports.getLatestVersion = exports.initializeNodeProject = exports.confirmRootDir = exports.run = void 0;
const node_fs_1 = __importDefault(require("node:fs"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const ansi_colors_1 = __importDefault(require("ansi-colors"));
const inquirer_1 = require("inquirer");
const setup_tools_1 = require("@nightwatch/setup-tools");
const constants_1 = require("./constants");
const logger_1 = __importDefault(require("./logger"));
const minimist_1 = __importDefault(require("minimist"));
const suggestSimilar_1 = __importDefault(require("./utils/suggestSimilar"));
const utils_1 = require("./utils");
const axios_1 = __importDefault(require("axios"));
const run = () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const argv = process.argv.slice(2);
        const _a = (0, minimist_1.default)(argv, {
            boolean: ['generate-config', 'native'],
            alias: {
                yes: 'y',
                browser: 'b',
                native: 'app'
            }
        }), { _: args } = _a, options = __rest(_a, ["_"]);
        // If string, convert options.browser to array.
        if (typeof options.browser === 'string') {
            options.browser = [options.browser];
        }
        // Filter flags that are not present in AVAILABLE_CONFIG_ARGS
        const wrongUserFlags = Object.keys(options).filter((word) => !constants_1.AVAILABLE_CONFIG_FLAGS.includes(word));
        if (wrongUserFlags.length > 0) {
            const findAndSuggestSimilarOption = (0, suggestSimilar_1.default)(wrongUserFlags[0], constants_1.AVAILABLE_CONFIG_FLAGS);
            if (findAndSuggestSimilarOption !== '') {
                logger_1.default.error(`error: unknown option '${wrongUserFlags[0]}'${findAndSuggestSimilarOption}`);
                return;
            }
        }
        let rootDir = path_1.default.resolve(process.cwd(), args[0] || '');
        logger_1.default.info(constants_1.NIGHTWATCH_TITLE);
        yield (0, exports.checkCreateNightwatchVersion)();
        if ((options === null || options === void 0 ? void 0 : options['generate-config']) && !(0, utils_1.isNodeProject)(rootDir)) {
            throw new Error(`package.json not found. Please run this command from your existing Nightwatch project.
      Or, use \`npm init nightwatch ${args[0] || '.'}\` to initialize a new Nightwatch project instead.`);
        }
        if (!args[0] && !(0, utils_1.isNodeProject)(rootDir) && node_fs_1.default.readdirSync(rootDir).length) {
            // `npm init nightwatch` is used and the cwd is not a node project
            // but contains some files (might be a mistake)
            rootDir = yield (0, exports.confirmRootDir)(rootDir);
        }
        if (!(0, utils_1.isNodeProject)(rootDir)) {
            (0, exports.initializeNodeProject)(rootDir);
        }
        const nightwatchInitiator = new setup_tools_1.NightwatchInitiator(rootDir, options);
        yield nightwatchInitiator.run();
    }
    catch (err) {
        logger_1.default.error(err);
        process.exit(1);
    }
});
exports.run = run;
const confirmRootDir = (rootDir) => __awaiter(void 0, void 0, void 0, function* () {
    logger_1.default.warn(`${ansi_colors_1.default.yellow('Warning:')} Current working directory is not a node project and already contains some files.`);
    const answers = yield (0, inquirer_1.prompt)([
        {
            type: 'list',
            name: 'confirm',
            message: 'Do you wish to initialize your Nightwatch project here?',
            choices: [
                { name: 'Yes', value: true },
                { name: 'No', value: false }
            ],
            default: false
        },
        {
            type: 'input',
            name: 'newRoot',
            message: 'Relative/absolute path to your existing/new project\'s root directory?',
            default: '.',
            when: (answers) => !answers.confirm
        }
    ]);
    // Insert a blank line after prompt.
    logger_1.default.info();
    if (answers.confirm) {
        return rootDir;
    }
    else {
        return path_1.default.resolve(rootDir, answers.newRoot);
    }
});
exports.confirmRootDir = confirmRootDir;
const initializeNodeProject = (rootDir) => {
    if (!node_fs_1.default.existsSync(rootDir)) {
        node_fs_1.default.mkdirSync(rootDir, { recursive: true });
    }
    logger_1.default.info(`${ansi_colors_1.default.yellow('package.json')} not found in the root directory. Initializing a new NPM project..\n`);
    (0, child_process_1.execSync)('npm init -y', {
        stdio: 'inherit',
        cwd: rootDir
    });
};
exports.initializeNodeProject = initializeNodeProject;
const getLatestVersion = () => __awaiter(void 0, void 0, void 0, function* () {
    return axios_1.default.get('https://registry.npmjs.org/create-nightwatch')
        .then(({ data }) => {
        return data['dist-tags'].latest;
    })
        .catch(() => null);
});
exports.getLatestVersion = getLatestVersion;
const checkCreateNightwatchVersion = () => __awaiter(void 0, void 0, void 0, function* () {
    const latestVersion = yield (0, exports.getLatestVersion)();
    const currentVersion = process.env.npm_package_version;
    if (latestVersion && currentVersion && latestVersion !== currentVersion) {
        logger_1.default.info(`We've updated this onboarding tool: ${ansi_colors_1.default.red(currentVersion)} -> ${ansi_colors_1.default.green(latestVersion)}. To get the latest experience, run: ${ansi_colors_1.default.green('npm init nightwatch@latest')}\n\n`);
    }
});
exports.checkCreateNightwatchVersion = checkCreateNightwatchVersion;
